' ImageToWeb: image generator sample module
' Copyright (C) 2006 by LudovicoVan (lv916@hotmail.com)
'
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

Public Module ImageToWeb

    ' Returns FALSE on error (invalid format)
    Public Function CheckFormat(ByVal ImgStream As IO.Stream) As Boolean
        CheckFormat = True
        Dim BmpSource As Bitmap
        Try
            BmpSource = New Bitmap(ImgStream)
        Catch SAEx As System.ArgumentException  ' Invalid image file on "New Bitmap"
            CheckFormat = False
        Finally                                 ' Cleanup
            If Not IsNothing(BmpSource) Then BmpSource.Dispose()
        End Try
    End Function

    Public Sub Generate(ByVal ImgStream As IO.Stream, ByVal ImgFileName As String, ByVal SizeBound As Integer)
        Dim BmpSource As Bitmap
        Dim BmpDest As Bitmap

        Try
            ' Get data from source image stream

            BmpSource = New Bitmap(ImgStream)

            ' Calculate size for destination image

            Dim SizeRatio As Double = SizeBound / Math.Max(BmpSource.Width, BmpSource.Height)

            Dim DestW As Integer = BmpSource.Width
            Dim DestH As Integer = BmpSource.Height

            If SizeRatio < 1 Then
                DestW = CInt(DestW * SizeRatio)
                DestH = CInt(DestH * SizeRatio)
            End If

            ' Create destination image

            BmpDest = New Bitmap(DestW, DestH)

            ' Set graphics modes for destination image and draw it

            Dim GrDest As Graphics = Graphics.FromImage(BmpDest)

            With GrDest
                .CompositingQuality = Drawing2D.CompositingQuality.HighQuality
                .SmoothingMode = Drawing2D.SmoothingMode.HighQuality
                .InterpolationMode = Drawing2D.InterpolationMode.HighQualityBicubic

                .DrawImage(BmpSource, 0, 0, DestW, DestH)
            End With

            ' Get codec infos (encoder) for destination image
            ' From KB article Q324790
            ' This requires the "image/jpeg" type be installed on the server

            Dim Encoder As Imaging.ImageCodecInfo
            For Each ICI As Imaging.ImageCodecInfo In Imaging.ImageCodecInfo.GetImageEncoders()
                If ICI.MimeType = "image/jpeg" Then
                    Encoder = ICI
                    Exit For
                End If
            Next

            ' Set encoder params

            Dim EncoderParams As New Imaging.EncoderParameters(3)
            EncoderParams.Param(0) = New Imaging.EncoderParameter(Imaging.Encoder.ColorDepth, 24)
            EncoderParams.Param(1) = New Imaging.EncoderParameter(Imaging.Encoder.Compression, 80)
            EncoderParams.Param(2) = New Imaging.EncoderParameter(Imaging.Encoder.Quality, 80)

            ' Save destination image

            BmpDest.Save(ImgFileName, Encoder, EncoderParams)

        Catch SAEx As System.ArgumentException  ' Invalid image file on "New Bitmap"
            Throw SAEx

        Finally                                 ' Cleanup
            If Not IsNothing(BmpDest) Then BmpDest.Dispose()
            If Not IsNothing(BmpSource) Then BmpSource.Dispose()
        End Try
    End Sub

End Module
